@@ -4,7 +4,7 @@ from docutils import core
 
 todo = [
     ('gui', 'gooeypy',
-        ['app', 'widget', 'styleset', 'effects']),
+        ['widget', 'styleset', 'effects']),
     ('widgets', 'gooeypy',
         ['label', 'button', 'input', 'slider', 'container', 'box', 'selectbox', 'textblock', 'image']),
     ]
