@@ -6,25 +6,22 @@ be sure to see)!
 
 import sys
 sys.path.insert(0, "..")
+import random
 
 import pygame
 import gooeypy as gui
 from gooeypy.const import *
 
+pygame.init()
+
 clock = pygame.time.Clock()
-
-# Here is our screen that we are already using in our game.
 screen = pygame.display.set_mode((640, 480))
-
-# Here is one thing we do different. We pass it our own surface that we want it
-# to use. This way, it won't set the display.
-gui.init(myscreen=screen)
-
-app = gui.App(width=640, height=100)
+myguiapp = gui.Container(width=640, height=100,
+        surface=pygame.surface.Surface((640, 100), pygame.SRCALPHA))
 
 
 # Lets create a very simple game where you move around an image.
-image = gui.get_image("image.png")
+image = pygame.image.load("image.png").convert_alpha()
 x = 200
 y = 250
 
@@ -45,8 +42,20 @@ def get_data():
 # see docs for full list of style options.
 l = gui.Label(get_data, align="center", y=70, font_size=25, color=(255,255,255))
 
-app.add(tb, l)
+# crab.gif is a color key. This demonstrates that GooeyPy uses it.
+crab_input = gui.Input(width=100, bgimage="crab.gif")
 
+myguiapp.add(tb, crab_input, l)
+
+# In your game you will probably have more than one gui area. Let's create a
+# second app and call it game_panel.
+game_panel = gui.Container(x=40, y=130,
+    surface=pygame.surface.Surface((180,50), pygame.SRCALPHA))
+
+button = gui.Button("Some button")
+game_panel.add(button)
+
+tick_count = 0
 while True:
     clock.tick(40)
 
@@ -72,11 +81,16 @@ while True:
     x = max(0, min(x, 460))
     y = max(100, min(y, 360))
 
-    app.run(events)
+    screen.fill((0,0,0))
+
+    myguiapp.run(events)
+    game_panel.run(events)
 
     # If you want the gui to be drawn to the screen every time, you
-    # can set "app.dirty = True" before calling app.draw()
-    app.draw()
+    # can set "myguiapp.dirty = True" before calling app.draw()
+    myguiapp.draw()
+    game_panel.dirty = True
+    game_panel.draw()
 
     # With updating the display you have a few options. If you want to flip the
     # entire screen, you can simply do:
@@ -91,11 +105,16 @@ while True:
     dirty_rect = screen.blit(image, (x,y))
     myupdaterects.append(dirty_rect)
 
+    dirty_rect = screen.blit(myguiapp.top_surface, myguiapp.pos)
+    myupdaterects.append(dirty_rect)
+
+    dirty_rect = screen.blit(game_panel.top_surface, game_panel.pos)
+    myupdaterects.append(dirty_rect)
+
     # Now we update the screen!
     pygame.display.update(myupdaterects)
 
     # Don't forget to clear the dirty rect list!
     myupdaterects = []
 
-    # Then we update the rects for the gui.
-    gui.update_display()
\ No newline at end of file
+    tick_count += 1
