@@ -9,12 +9,14 @@ import pygame
 import gooeypy as gui
 from gooeypy.const import *
 
+pygame.init()
+
 # Ok, this is really really cool... a CLOCK!!! Ok, so maybe that's not so new...
 clock = pygame.time.Clock()
 
-gui.init(640, 480)
+screen = pygame.display.set_mode((640, 480), pygame.SRCALPHA)
 
-app = gui.App(width=640, height=480)
+app = gui.Container(width=640, height=480, surface=screen)
 
 
 class Menus(gui.Container):
@@ -83,11 +85,16 @@ while not quit:
         if event.type == QUIT:
             quit = True
 
+        elif event.type == KEYDOWN:
+            if event.key == K_ESCAPE:
+                sys.exit()
+
+
     app.run(events)
     app.draw()
 
-    gui.update_display()
+    pygame.display.flip()
 
 # Now, was that very hard? Look how little code it took to get a menu system
 # setup! This is a big goal for GooeyPy, to be able to get what it needs to get
-# done with as little code as possible.
\ No newline at end of file
+# done with as little code as possible.
