@@ -1,28 +1,30 @@
 import pygame
-import util
 from const import *
-import container, widget
-
 from cellulose import *
+import container
+import widget
 
 
 class _box(container.Container):
 
-    expand = util.ReplaceableCellDescriptor()
+    expand = widget.ReplaceableCellDescriptor()
 
     def __init__(self, expand=True, **params):
         container.Container.__init__(self, **params)
 
         self.expand = expand
 
-    def add(self, widgets):
-        container.Container.add(self, widgets)
+    def add(self, *widgets):
+        super(_box, self).add(*widgets)
         self.position_widgets()
 
     def remove(self, widgets):
         container.Container.remove(self, widgets)
         self.position_widgets()
 
+    def draw(self, drawbg=True):
+        self.dirty = True
+        container.Container.draw(self, drawbg)
 
 class VBox(_box):
     """
@@ -45,7 +47,7 @@ class VBox(_box):
             # I shouldn't have to do this...
             w.dirty = True
 
-    def add(self, widgets):
+    def add(self, *widgets):
         """
         VBox.add(widgets) -> None
         ^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -56,15 +58,16 @@ class VBox(_box):
         widgets
             can ether be a single widget or a list of widgets.
         """
-        if type(widgets) != list:
-            widgets = [widgets]
 
-        _box.add(self, widgets)
+        super(VBox, self).add(*widgets)
         if self.expand:
             width = self.width
             def width_func():
-                return width - (self.style_option["padding-left"] +\
+                w = width - (self.style_option["padding-left"] +\
                         self.style_option["padding-right"])
+                if self.scrollable:
+                    w -= self.vslider.width
+                return w
             for w in widgets:
                 w._cells['width'] = ComputedCell(width_func)
 
@@ -81,14 +84,18 @@ class VBox(_box):
     width = ComputedCellDescriptor(width)
 
     def height(self):
+        return widget.Widget.height.function(self, self.content_height)
+    height = ComputedCellDescriptor(height)
+
+    def content_height(self):
         height = 0
         for w in self.widgets:
             height += w.height + self.style_option["spacing"]
         height -= self.style_option["spacing"]
 
         height += self.style_option["padding-top"] + self.style_option["padding-bottom"]
-        return widget.Widget.height.function(self, height)
-    height = ComputedCellDescriptor(height)
+        return height
+    content_height = ComputedCellDescriptor(content_height)
 
 
 class HBox(_box):
@@ -112,9 +119,9 @@ class HBox(_box):
             w.dirty = True
 
 
-    def add(self, widgets):
+    def add(self, *widgets):
         """
-        VBox.add(widgets) -> None
+        HBox.add(widgets) -> None
         ^^^^^^^^^^^^^^^^^^^^^^^^^
         Add widgets to Container.
 
@@ -123,10 +130,8 @@ class HBox(_box):
         widgets
             can ether be a single widget or a list of widgets.
         """
-        if type(widgets) != list:
-            widgets = [widgets]
 
-        _box.add(self, widgets)
+        super(HBox, self).add(*widgets)
         if self.expand:
             height = self.height
             def height_func():
