@@ -2,7 +2,6 @@
 <title>Effects appliable to widgets</title>
 <h1>Effects</h1>
 """
-import util
 
 
 try:
@@ -51,4 +50,4 @@ class Pulsate:
             for s  in self.used_surfs:
                 s.set_alpha(self.orig_alpha)
             return False
-        return surf
\ No newline at end of file
+        return True
