@@ -1,20 +1,20 @@
 import pygame
 from pygame.locals import *
-import util
 from const import *
 
 from cellulose import *
-from cellulose.restrictions import StringRestriction
+from cellulose.extra.restrictions import StringRestriction
 
+import widget
 
-class Label(util.widgets.Widget):
+class Label(widget.Widget):
     """
     Label(value) -> Label widget
     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
     Basic widget for simply displaying some text. Not really much to
     it, you can pass it a value which it will display.
     """
-    value = util.ReplaceableCellDescriptor(restriction=StringRestriction())
+    value = widget.ReplaceableCellDescriptor(restriction=StringRestriction())
 
     def __init__(self, value, **params):
         super(Label,self).__init__(self, **params)
@@ -27,7 +27,7 @@ class Label(util.widgets.Widget):
         ^^^^^^^^^^^^^^^^^^^^^^^^^
         The font that this label is using.
         """
-        f = util.app.get_font(self.style_option["font-family"], self.style_option["font-size"])
+        f = self.get_font(self.style_option["font-family"], self.style_option["font-size"])
         if self.style_option["font-weight"] == "bold":
             f.set_bold(True)
         return f
@@ -35,12 +35,12 @@ class Label(util.widgets.Widget):
 
     def width(self):
         w = self.font.size(self.value)[0]
-        return util.widgets.Widget.width.function(self,w)
+        return widget.Widget.width.function(self,w)
     width = ComputedCellDescriptor(width)
 
     def height(self):
         h = self.font.size(self.value)[1]
-        return util.widgets.Widget.height.function(self,h)
+        return widget.Widget.height.function(self,h)
     height = ComputedCellDescriptor(height)
 
     def font_x(self):
@@ -63,13 +63,13 @@ class Label(util.widgets.Widget):
 
 
     def font_value(self):
-        return self.font.render(self.value, self.style_option["antialias"],
-                self.style_option["color"])
+        return self.font.render(self.value,
+                                self.style_option["antialias"],
+                                self.style_option["color"])
     font_value = ComputedCellDescriptor(font_value)
 
     def draw_widget(self):
         # We do this incase we are trying to blit the text into a space smaller
         # than it can fit into.
 
-        util.blit(self.font_value, (self.font_x,self.font_y),
-                clip_rect=self.clip_rect)
\ No newline at end of file
+        self.blit(self.font_value, (self.font_x,self.font_y), clip_rect=self.clip_rect)
