@@ -1,21 +1,18 @@
 import pygame
-import util
 from const import *
-import widget
 
 from cellulose import *
 
-
+import widget
 
 class _slider(widget.Widget):
 
-    _value = util.ReplaceableCellDescriptor()
-    length = util.ReplaceableCellDescriptor()
-    min_value = util.ReplaceableCellDescriptor()
-    step = util.ReplaceableCellDescriptor()
+    _value = widget.ReplaceableCellDescriptor()
+    length = widget.ReplaceableCellDescriptor()
+    min_value = widget.ReplaceableCellDescriptor()
+    step = widget.ReplaceableCellDescriptor()
 
-    def __init__(self, value=0, min_value=0, length=20,
-                step=True, **params):
+    def __init__(self, value=0, min_value=0, length=20, step=True, **params):
         widget.Widget.__init__(self, **params)
         self.step = step
         self.length = length
@@ -23,7 +20,7 @@ class _slider(widget.Widget):
         self.value = value
 
         # Create the marker widget and link it's values to sliders.
-        self.marker = widget.Widget()
+        self.marker = Marker()#widget.Widget()
         self.marker.parent = self
         self.marker._cells["hovering"] = self._cells["hovering"]
         self.marker._cells["focused"] = self._cells["focused"]
@@ -74,8 +71,8 @@ class HSlider(_slider):
         Otherwise it won't round off the decimal.
     """
 
-    def __init__(self, **params):
-        _slider.__init__(self, **params)
+    def __init__(self, theme="default", surface=None, **params):
+        _slider.__init__(self, theme=theme, surface=surface, **params)
 
         def width_func():
             return self.height-(self.style_option["padding-left"]+\
@@ -92,6 +89,7 @@ class HSlider(_slider):
         self.marker.draw()
 
     def event(self, e):
+        widget.Widget.event(self, e)
         adj = 0
         if e.type == MOUSEBUTTONDOWN:
             if not self.marker.rect.collidepoint(e.pos):
@@ -138,6 +136,7 @@ class VSlider(_slider):
         self.marker.draw()
 
     def event(self, e):
+        widget.Widget.event(self, e)
         adj = 0
         if e.type == MOUSEBUTTONDOWN:
             if not self.marker.rect.collidepoint(e.pos):
@@ -156,4 +155,8 @@ class VSlider(_slider):
             if v > self.value:
                 self.value += 1
             else:
-                self.value -= 1
\ No newline at end of file
+                self.value -= 1
+
+class Marker(widget.Widget):
+    def __init__(self):
+        widget.Widget.__init__(self)
