@@ -1,13 +1,16 @@
 """ <title>Styling options</title> """
-import util
+import os
 import pygame
 
 from cellulose import *
 
+from descriptor import ReplaceableCellDescriptor
+
 class Style:
     def __init__(self, styleset, values):
         self.styleset = styleset
         self.widget = styleset.widget
+        self.path = self.widget.path
         self.values = values
         self.setup(values)
 
@@ -17,6 +20,66 @@ class Style:
     def render(self):
         pass
 
+    def draw_rect(self, surf, color, rect, width=0):
+        pygame.draw.rect(surf, color, rect, width)
+
+    def get_image_path(self, path):
+        if os.path.isfile(os.path.join(self.path, path)):
+            path = os.path.join(self.path, path)
+        return path
+
+    def get_image(self, path):
+        if type(path) != str:
+            image = pygame.image.load(path)
+            if image.get_colorkey():
+                return image.convert()
+            else:
+                return image.convert_alpha()
+        path = self.get_image_path(path)
+        return self.really_get_image(path)
+
+    imagestore = {}
+    def really_get_image(self, path, nocache=False):
+        """ really_get_image(path, [colorkey, [nocache]]) -> pygame.Surface
+
+        path should be a sting. Example: really_get_image("data/image/my_image.png")
+        Use forward slashed as demonstrated even if you are developing under
+        windows, it will open the files os independent. """
+
+
+        if Style.imagestore.has_key(path):
+            return Style.imagestore[path]
+        else:
+            key = path.split("/")
+            if len(key) == 1:
+                key = key[0]
+            else:
+                p = ""
+                for k in key:
+                    p = os.path.join(p, k)
+                key = p
+            if path[0] == "/":
+                # HACK: shouldn't have to do this... (and won't work on windows)
+                key = "/"+key
+            if not os.path.isfile(key):
+                key = os.path.join(os.path.dirname(__file__), key)
+                if not os.path.isfile(key):
+                    print key
+                    print "could not find image"
+                    sys.exit()
+            image = pygame.image.load(key)
+            if image.get_colorkey():
+                image = image.convert()
+            else:
+                image = image.convert_alpha()
+
+            # Add it so it doesn't have to create surf again for this image.
+            if not nocache:
+                Style.imagestore.update({path: image})
+
+        return image
+
+
 
 class Opacity(Style):
     def setup(self, values):
@@ -27,9 +90,11 @@ class Opacity(Style):
         self.styleset.applied_styles["opacity"] = self
 
     def render(self):
+        self.styleset.surface.set_alpha(None)
         self.styleset.surface.set_alpha(self.values)
 
 
+
 class BgColor(Style):
     def setup(self, values):
         self.color = None
@@ -55,7 +120,7 @@ class BgColor(Style):
             self.rect.width -= w*2
             self.rect.height -= w*2
 
-        util.draw_rect(self.styleset.surface, self.color, self.rect)
+        self.draw_rect(self.styleset.surface, self.color, self.rect)
 
 
 class Border(Style):
@@ -66,13 +131,14 @@ class Border(Style):
         self.styleset.applied_styles["border"] = self
 
     def render(self):
-        util.draw_rect(self.styleset.surface, self.color, self.styleset.surface.get_rect(), self.width)
+        self.draw_rect(self.styleset.surface, self.color, self.styleset.surface.get_rect(), self.width)
 
 
 class BgImage(Style):
     def setup(self, values):
         if values[0] != "none":
-            self.im = util.app.get_image(values[0])
+            self.im = self.get_image(values[0])
+            self.colorkey = self.im.get_colorkey()
             self.style = None
             if len(values) > 1:
                 self.style = values[1]
@@ -126,13 +192,14 @@ class BgImage(Style):
             num_x = self.styleset.surface.get_width()  / t.get_width() + 1
             num_y = self.styleset.surface.get_height() / l.get_height() + 1
 
-            pygame.draw.rect(self.styleset.surface, c, self.styleset.surface.get_rect())
-            for x in xrange(num_x):
-                self.styleset.surface.blit(t, (x*t.get_width(), 0))
-                self.styleset.surface.blit(b, (x*t.get_width(), self.styleset.surface.get_height()-t.get_height()))
-            for y in xrange(num_y):
-                self.styleset.surface.blit(l, (0, y*l.get_height()))
-                self.styleset.surface.blit(r, (self.styleset.surface.get_width()-r.get_width(), y*r.get_height()))
+            center_rect = pygame.Rect(w, h, self.styleset.surface.get_rect().w - 2 * w, self.styleset.surface.get_rect().h - 2 * h)
+            pygame.draw.rect(self.styleset.surface, c, center_rect)
+            for x in xrange(w, self.styleset.surface.get_width() - w, t.get_width()):
+                self.styleset.surface.blit(t, (x, 0))
+                self.styleset.surface.blit(b, (x, self.styleset.surface.get_height()-t.get_height()))
+            for y in xrange(h, self.styleset.surface.get_height() - h, l.get_height()):
+                self.styleset.surface.blit(l, (0, y))
+                self.styleset.surface.blit(r, (self.styleset.surface.get_width()-r.get_width(), y))
             self.styleset.surface.blit(tl, (0,0))
             self.styleset.surface.blit(bl, (0,self.styleset.surface.get_height()-h))
             self.styleset.surface.blit(tr, (self.styleset.surface.get_width()-w,0))
@@ -202,11 +269,6 @@ class StyleSet(object):
 
         mywidget.stylesets["hover"]["color"] = (255,10,10)
 
-    **WARNING:** Do not use the color (0,0,0)! It is used as the surface color
-    key. That means that if you use it it won't show up! If you want to use the
-    color black, use (5,5,5) or something (which is indistinguishable from pure
-    black). This applies to font colors sometimes. :P
-
     Style options
     -------------
 
@@ -230,7 +292,7 @@ class StyleSet(object):
     position         relative | absolute
     bgcolor          (200,0,0)
     bgimage          image.png repeat|repeat-x|repeat-y|no-repeat|slice
-    border           (0,250,20) 3
+    border           3 (0,250,20)
     spacing          12 # For certain container widgets.
     opacity          25 # Alpha value. Valid otpions between 0 and 255.
     antialias        1 | 0
@@ -257,8 +319,10 @@ class StyleSet(object):
         Another styleset that this styleset should inherit from.
     """
 
-    parent = util.ReplaceableCellDescriptor()
-    widget = util.ReplaceableCellDescriptor()
+    parent = ReplaceableCellDescriptor()
+    widget = ReplaceableCellDescriptor()
+    has_opacity = ReplaceableCellDescriptor()
+    colorkey = ReplaceableCellDescriptor()
 
     # Setting a default style option here will make it use it at all times
     # unless explicitly set. In other words, inheriting doesn't work for these
@@ -271,8 +335,8 @@ class StyleSet(object):
         "spacing":0,
         "font-weight":"normal",
         "effect":"none",
-        "width":util.screen_width,
-        "height":util.screen_height,
+        "width":0,
+        "height":0,
         "x":0,
         "y":0,
         "position":"relative",
@@ -293,6 +357,9 @@ class StyleSet(object):
         self.parent = parent
         self._applied_styles = CellDict(self.default_styles)
 
+        self.has_opacity = False
+        self.colorkey = None
+
         def __getitem__(v):
             return self.__getitem__(v)
         self._applied_styles.__getitem__ = __getitem__
@@ -312,8 +379,18 @@ class StyleSet(object):
             self.surface = self.parent.surf.copy()
         else:
             size = (int(self.widget.width), int(self.widget.height))
-            self.surface = pygame.Surface(size)
-            self.surface.set_colorkey((0,0,0,0))
+            if self.colorkey:
+                self.surface = pygame.Surface(size)
+                self.surface.fill(self.colorkey)
+            else:
+                self.surface = pygame.Surface(size, pygame.SRCALPHA)
+
+        # Surfaces that use opacity have to turn off the per-pixel alphas
+        if self.has_opacity:
+            self.surface = self.surface.convert()
+        if self.colorkey:
+            self.surface = self.surface.convert()
+            self.surface.set_colorkey(self.colorkey)
 
         for s in self.applied_styles.values():
             if hasattr(s, "render"):
@@ -356,6 +433,8 @@ class StyleSet(object):
             self.styles[o](self, values)
         else:
             self.generic(option, values)
+        if o == 'opacity':
+            self.has_opacity = True
 
 
     def generic(self, n, v):
@@ -367,4 +446,4 @@ class StyleSet(object):
                     v = int(v)
                 except ValueError:
                     pass
-        self.applied_styles[n] = v
\ No newline at end of file
+        self.applied_styles[n] = v
