@@ -1,17 +1,23 @@
 """ <title>Base widget</title> """
 import pygame
-import util
 from const import *
 import styleset
 import effects
+import os
 import sys
+import traceback
 
+import cellulose
 from cellulose import *
+from cellulose.extra.restrictions import *
+
+from descriptor import ReplaceableCellDescriptor
+import styleset
 
 class Widget(DependantCell):
     """
-    Widget([disabled, [active, [parent, [custom_styles]]]]) -> Widget
-    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+    Widget([disabled, [active, [parent, [theme, [surface, [custom_styles]]]]]]) -> Widget
+    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
     Base widget class.
 
@@ -66,6 +72,14 @@ class Widget(DependantCell):
     parent
         This is used internally when creating new widget types.
 
+    theme
+        The theme name or path to theme that you want this widget to use.
+
+    surface
+        The surface you want this widget to blit onto. This is a required
+        argument for top-level widgets (widgets without a parent). This will
+        override blitting to a parent (if it has one).
+
     custom_styles
         When you pass any extra arguments they get passed as a
         custom style; the key being the style option and the
@@ -78,19 +92,40 @@ class Widget(DependantCell):
     """
 
 
-    disabled  = util.ReplaceableCellDescriptor()
-    hovering  = util.ReplaceableCellDescriptor()
-    focused   = util.ReplaceableCellDescriptor()
-    stylesets = util.ReplaceableCellDescriptor()
+    disabled  = ReplaceableCellDescriptor()
+    hovering  = ReplaceableCellDescriptor()
+    focused   = ReplaceableCellDescriptor()
+    stylesets = ReplaceableCellDescriptor()
     #parent    = InputCellDescriptor()
-    container = util.ReplaceableCellDescriptor()
-    pressing  = util.ReplaceableCellDescriptor()
-    selected  = util.ReplaceableCellDescriptor()
-    active    = util.ReplaceableCellDescriptor()
+    container = ReplaceableCellDescriptor()
+    pressing  = ReplaceableCellDescriptor()
+    selected  = ReplaceableCellDescriptor()
+    active    = ReplaceableCellDescriptor()
 
-    def __init__(self, disabled=False, active=True, parent=None, **params):
+    def __init__(self, disabled=False, active=True, parent=None, theme=None,
+            surface=None, **params):
         DependantCell.__init__(self)
 
+        self.debug = False
+
+        self.top_surface = surface
+
+        self.theme = {}
+
+        # If the theme to use is not specified try to inherit from parent widget
+        if theme:
+            self.theme_name = theme
+            self.load_theme(self.theme_name)
+        else:
+            if parent:
+                self.theme_name = parent.theme_name
+                self.theme = parent.theme
+                # FIXME path is rather ambiguous - theme_path?
+                self.path = parent.path
+            else:
+                self.theme_name = "default"
+                self.load_theme(self.theme_name)
+
         self.hovering = False
         self.focused = False
         self.active = active
@@ -118,8 +153,12 @@ class Widget(DependantCell):
         # This widget is still drawn if disabled is True, it just doesn't work.
         self.disabled = disabled
 
-
-        self.style_option = ComputedDict(lambda key: self.style[key])
+        def get_style_option(key):
+            s = self.style[key]
+            if hasattr(s, "values"):
+                return s.values
+            return s
+        self.style_option = ComputedDict(get_style_option)
 
         # FIXME: Adding one more causes a really wierd bug where sometimes a
         # widget will inherit from Widget styling when not using default style.
@@ -129,9 +168,8 @@ class Widget(DependantCell):
         # Generate the stylesets.
         surf = None
         style = None
-        s = util.app.get_style(self)
+        s = self.get_style()
 
-        #FIXME: This needs to be run every time the parent changes?
         if s:
             #TODO: clear unnecessary spaces.
             for cmd in s.split(";"):
@@ -294,12 +332,17 @@ class Widget(DependantCell):
                     y += (self.parent.height - self.height)
                 else:
                     y += int(self.parent.stylesets["default"]["padding-top"])
-            if self.container:
+            if self.container and self.container.scrollable:
                 x -= self.container.offset_x
                 y -= self.container.offset_y
         return (x,y)
     pos = ComputedCellDescriptor(pos)
 
+    def top_level_parent(self):
+        if not self.parent:
+            return self
+        return self.parent.top_level_parent
+    top_level_parent = ComputedCellDescriptor(top_level_parent)
 
     def width(self, w=None):
         """
@@ -309,9 +352,10 @@ class Widget(DependantCell):
         padding (for widgets that contain other widgets), min-width and
         max-width styles.
         """
-        if w is None or self.style_option["width"]:
+        if not w or self.style_option["width"]:
             w = self.style_option["width"] + self.style_option["padding-left"]+\
                     self.style_option["padding-right"]
+            w = max(w,0)
         width = w
         min_width = self.style_option["min-width"]
         max_width = self.style_option["max-width"]
@@ -334,6 +378,7 @@ class Widget(DependantCell):
         if h is None or self.style_option["height"]:
             h = self.style_option["height"] + self.style_option["padding-top"]+\
                     self.style_option["padding-bottom"]
+            h = max(h,0)
         height = h
         min_height = self.style_option["min-height"]
         max_height = self.style_option["max-height"]
@@ -385,14 +430,14 @@ class Widget(DependantCell):
     surface = ComputedCellDescriptor(surface)
 
 
-    def draw(self):
+    def draw(self, drawbg=True):
         """
         Widget.draw() -> None
         ^^^^^^^^^^^^^^^^^^^^^
         draws the widget if necessary.
         """
 
-        if self.dirty or self.effect or self.style_option["effect"] != ['none']:
+        if self.dirty or self.effect or self.style_option["effect"] != 'none':
             # We don't want to keep around old dependencies. We gotta clear them
             # before adding the new ones.
             for dep in self.dependencies.values():
@@ -401,41 +446,70 @@ class Widget(DependantCell):
 
             self.push()
             try:
+                # HACK: the pulsate effect sets opacity values on the surface
+                # outside of the StyleSet framework, so we have to tell it
+                # that it can't have per-pixel alphas anymore when we get the
+                # surface.  Unfortunately, this is permanent.  We can't go
+                # back to per-pixel alphas.
+                if self.style_option["effect"][0] == "pulsate":
+                    self.style.has_opacity = True
                 surf = self.style.surf
+
+                # Fill the main surface if this widget has it before drawing
+                # anything else
+                if self.top_surface:
+                    self.top_surface.fill((0,0,0,0))
+
                 if self.effect:
                     # An effect is running.
-                    r = self.effect.run(surf)
-                    if not r:
+                    if not self.effect.run(surf):
                         self.effect = None
-                    else:
-                        surf = r
+
                 elif self.style_option["effect"][0] == "pulsate":
                     i = int(self.style_option["effect"][1])
                     self.effect = effects.Pulsate(i)
-                    surf = self.effect.run(surf)
-
-                #print self.__class__.__name__
+                    self.effect.run(surf)
 
                 x,y = self.pos
                 # FIXME: won't work if widget doesn't have container... but
                 # fixes the x=y=0 display bug.
-                if self._last_blit_rect and self.container:
+                if drawbg and self._last_blit_rect and self.container:
                     lbrpos =(self._last_blit_rect.x, self._last_blit_rect.y)
                     self._last_blit_rect.x -= self.container.x
                     self._last_blit_rect.y -= self.container.y
-                    util.blit(self.container.surface, lbrpos,
-                            clip_rect=self._last_blit_rect)
+                    self.blit(self.container.surface, lbrpos, self._last_blit_rect)
 
-                #clip_rect = surf.get_rect()
-                #if self.container and self.style_option["position"] == "relative":
-                    #clip_rect = self.container.rect
-                    #clip_rect.x = self.container.offset_x
-                    #clip_rect.y = min(self.container.offset_y, self.y)
-                    #x += clip_rect.x
-                    #y += clip_rect.y
+                clip_rect = surf.get_rect()
+                if self.container and self.container.scrollable:
+                    #y -= self.container.offset_y
+                    if self.container.offset_y > self.y+self.height:
+                        return
+                    if self.container.offset_y+self.container.height < self.y+\
+                            self.container.style_option["padding-bottom"]+\
+                            self.container.style_option["padding-top"]+\
+                            self.height:
+                        # Off the  bottom. Need to figure out how to do clipping
+                        return
+
+                    #clip_rect = surf.get_rect()#self.container.rect
+                    clip_rect.x = 0#min(self.x-self.container.offset_x, self.x)
+                    clip_rect.y = min(0, self.y-self.container.offset_y)
+
+                    #y = min(y, self.container.height-self.height)
+
+                    #if self.container.offset_y+self.container.height < self.y+\
+                            #self.height+\
+                            #self.container.style_option["padding-bottom"]+\
+                            #self.container.style_option["padding-top"]:
+                        #clip_rect.height = self.container.height-\
+                                #self.container.style_option["padding-bottom"] - self.y
+                        #print self.y
+                self.clip_rect = clip_rect
+
+                self._last_blit_rect = self.blit(surf, (x, y), clip_rect)
 
-                self._last_blit_rect = util.blit(surf, (x, y))
                 self.draw_widget()
+
                 self.dirty = False
             finally:
                 self.pop()
@@ -448,8 +522,12 @@ class Widget(DependantCell):
         """
         pass
 
+    def repetitive_events(self):
+        pass
 
     def event(self, e):
+        if self.focused:
+            self.repetitive_events()
         pass
 
 
@@ -543,3 +621,134 @@ class Widget(DependantCell):
         if send:
             self.send(e.type, e)
             self.event(e)
+
+
+    def blit(self, surf, (x,y), clip_rect=None):
+        if not surf: return
+
+        if self.parent == self.top_level_parent:
+            # In case the top parent's origin is not 0,0 we need to compensate
+            # with an offset.
+            # Due to the nature of blitting we only want to offset on the top
+            # level widget's direct children.
+            dx_offset, dy_offset = self.top_level_parent.pos
+            x -= dx_offset
+            y -= dy_offset
+
+        # FIXME: this will make the top-level widget draw properly but not it's
+        # children.
+        #elif self.top_surface:
+            #dx_offset, dy_offset = self.pos
+            #x -= dx_offset
+            #y -= dy_offset
+
+        if not clip_rect:
+            clip_rect = surf.get_rect()
+        if not self.top_surface:
+            if self.container:
+                return self.container.blit(surf, (x,y), clip_rect)
+            elif self.parent:
+                return self.parent.blit(surf, (x,y), clip_rect)
+            else:
+                raise ValueError(str(self)+" is not a top level element and does not have a surface to blit to. Set `Widget.top_surface` or pass the `surface` when creating the widget.")
+
+        rect = self.top_surface.blit(surf, (x,y), clip_rect)
+        return rect
+
+    def run(self, events):
+        # FIXME Will this work for the top element itself?
+        if self.parent:
+            # In case the top parent's origin is not 0,0 we need to compensate
+            # with an offset.
+            dx_offset, dy_offset = self.top_level_parent.pos
+        else:
+            dx_offset = dy_offset = 0
+        for c in ReplaceableCellDescriptor.custom_cells:
+            c.dependency_changed()
+        cellulose.default_observer_bank.flush()
+        for e in events:
+            if (dx_offset or dy_offset) and (e.type == MOUSEBUTTONDOWN or e.type == MOUSEBUTTONUP or e.type == MOUSEMOTION):
+                if e.type == MOUSEMOTION:
+                    e = pygame.event.Event(e.type, buttons=e.buttons, rel=e.rel, pos=(e.pos[0] - dx_offset, e.pos[1] - dy_offset))
+                else:
+                    e = pygame.event.Event(e.type, button=e.button, pos=(e.pos[0] - dx_offset, e.pos[1] - dy_offset))
+            # Normally should call _event... but that makes it run a bit sluggish.
+            self.event(e)
+
+
+    def get_style(self):
+        w = self
+        if self.theme.has_key(w.__class__.__name__):
+            t = self.theme[w.__class__.__name__]
+            n = w.__class__.__name__
+            while True:
+                if w.parent:
+                    w = w.parent
+                    if self.theme.has_key(w.__class__.__name__+" "+n):
+                        n = w.__class__.__name__+" "+n
+                        t += self.theme[n]
+                else:
+                    break
+            return t
+        else:
+            return ""
+
+
+    def get_font(self, file, size):
+        size = int(size)
+        return pygame.font.Font(os.path.join(self.path, file), size)
+
+
+    def load_theme(self, theme_name):
+        if os.path.isfile(theme_name):
+          cfile = open(theme_name)
+          self.path = os.path.dirname(theme_name)
+        else:
+          paths = []
+          paths.append(os.path.join("data", "themes", theme_name))
+          paths.append(os.path.join(os.path.dirname(__file__), "data", "themes", theme_name))
+          paths.append(os.path.join("..", "data", "themes", theme_name))
+          paths.append(os.path.join("..", "..", "data", "themes", theme_name))
+
+          self.path = None
+          p = None
+          for p in paths:
+              if os.path.isdir(p):
+                  self.path = p
+                  break
+          if not self.path:
+              print "Failed to find theme!"
+              sys.exit()
+
+          cfile = open(os.path.join(self.path, "theme.style"))
+
+        ls = cfile.readlines()
+
+        keys = None
+        v = ""
+
+        for l in ls:
+            l = l.replace('\n', '')
+            l = l.strip()
+            if not l or l[0] == "#": continue
+            if l[-1] == "{":
+                # A definition.
+                l = l[0:-2]
+                keys = l.split(",")
+            elif l == "}":
+                # End of section.
+                for o in keys:
+                    o = o.strip()
+                    if not self.theme.has_key(o):
+                        self.theme[o] = v
+                    else:
+                        self.theme[o] += v
+                v = ""
+                keys = None
+            else:
+                v += l
+
+        cfile.close()
+
+
+
